
import java.awt.Dimension;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingWorker;

public class MyPanel extends Panel {

    private final int REFRESH_PERIOD = 10;
    private Simulator sim;
    private MyCanvas canvas;
    private JTextField statusBar;

    private class StartSimListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            if (!sim.isRunning()) {
                sim.start();
                statusBar.setText("Sim started");
                SwingWorker<Object, Object> swingWorker = new SwingWorker<Object, Object>() {
                    @Override
                    protected Object doInBackground() throws Exception {
                        Object obj = new Object();
                        while (sim.isRunning()) {
                            if (sim.isChanged()) {
                                canvas.redraw();
				//Toolkit.getDefaultToolkit().sync();
                            }
                            try {
                                Thread.sleep(REFRESH_PERIOD);
                            } catch (InterruptedException e) {
                            }
                        }
                        return null;
                    }
                };
                swingWorker.execute();
            } else {
                statusBar.setText("Sim is already running");
            }
        }
    }

    private class StopSimListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            sim.shutdown();
            statusBar.setText("Computation stopped");
        }
    }

    public MyPanel(Simulator sim) {
        this.sim = sim;
        createComponents();
    }

    private void createComponents() {
        JPanel controlButtonsPanel = new JPanel();
        createControlButtons(controlButtonsPanel);
        canvas = new MyCanvas(sim.getWorld().getDimension());

        SimulatorGUI simGui = new SimulatorGUI(sim, canvas);
        simGui.createObjects();

        statusBar = createStatusBar("Waiting for your commands");
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        add(controlButtonsPanel);
        add(canvas);
        add(statusBar);
    }

    private JPanel createControlButtons(JPanel panel) {
        JButton btnCompute = new JButton("Start");
        JButton btnStop = new JButton("Stop");
        JButton btnQuit = new JButton("Quit");

        btnCompute.addActionListener(new StartSimListener());
        btnStop.addActionListener(new StopSimListener());
        btnQuit.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                System.out.println("Force quit");
                System.exit(0);
            }
        });

        panel.add(btnCompute);
        panel.add(btnStop);
        panel.add(btnQuit);
        return panel;
    }

    private JTextField createStatusBar(String initMessage) {
        JTextField statusBar = new JTextField();
        statusBar.setEditable(false);
        statusBar.setText(initMessage);
        statusBar.setHorizontalAlignment(JTextField.LEFT);
        statusBar.setMaximumSize(
                new Dimension(
                        Integer.MAX_VALUE,
                        statusBar.getPreferredSize().height
                ));
        return statusBar;
    }
}
